/*
 * This file is part of Mutator.
 *
 * Mutator is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mutator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mutator.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * <-[ Mutator v0.1 ]->
 * 
 * Autor: @AloneInTheShell
 * Mail: alone.in.the.shell@gmail.com
 *
 * Este soft ha sido creado ante la imposiblidad de encontrar un 
 * mutador de diccionarios que se adaptara a mis necesidades, con lo
 * que posiblemente NO cumpla las tuyas ;P
 * Por defecto realiza todas las mutaciones definidas y segun las
 * opciones que pongas deshabilitas unas u otras.
 * Una de las principales caracteristicas frente a otros mutadores
 * de diccionarios es que aplica determinadas "mutaciones" a los
 * resultados de pruebas anteriores, permitiendo generar passwords
 * del tipo: 
 * corporation -> C0rp0r4t10n_2012
 * Lo cual no he encontrado en ningun otro programa del estilo.
 *
 * Usage:
 * $ mutator [options] wordlist
 *      -v, --version       Program version
 *      -h, --help          This help output
 *      -o, --output        File to write the output
 *      -b, --basic         Only "case" and "l33t" mutations
 *      -a, --advanced      Only advanced mutations
 *      -nd, --no-date      No append,prepend date 
 *      -ns, --no-special   No append specials chars 
 *      -nc, --no-strings   No append,prepend hardcoded strings
 *      -i, --include*       Include the original words in the 
 *                          resulting file
 */

#include "main.h"

int main (int argc, char ** argv)
{
    s_files * fdstruct;
    char output[WORDSIZE];
    char input[WORDSIZE];
    int opt;
    usi has_output=FALSE;//,no_year;
//    usi no_special,no_int,no_strings;

    memset(output, '\0', WORDSIZE);
    memset(input, '\0', WORDSIZE);

    fdstruct = (s_files *)malloc(sizeof(s_files));

    if(fdstruct == NULL)
    {
        printf("ERROR: no memory is available!!!");
        exit(-1);
    }

    memset(fdstruct, '\0',sizeof(s_files));

    extern char *optarg;
    extern int optind;

    while((opt = getopt(argc, argv, ":vo:badsich")) != -1)
    {
        switch(opt)
        {
            case 'v':
                printf("%s %s - by %s\n",PROGRAM,VERSION,AUTHOR);
                exit(-1);
                break;
            case 'o':
                if (optarg != 0)
                {
                    has_output=TRUE;
                    strncpy(output, optarg, MIN(strlen(optarg),WORDSIZE));
                }
                break;
            case 'b':
                do_basic=TRUE;
                break;
            case 'a':
                do_adv=TRUE;
                break;
            case 'd':
                //no_year=TRUE;
                break;
            case 's':
                //no_special=TRUE;
                break;
            case 'i':
                //no_int=TRUE;
                break;
            case 'c':
                //no_strings=TRUE;
                break;
            case ':':
            case '?':
                printf("Unknow option or no option argument specificed.\n");
            default :
            case 'h':
                usage();
                break;
        }
    }

    if (!has_output)
        strncpy(output, OUTPUT, strlen(OUTPUT));

    if (argv[optind]!=NULL)
        strncpy(input, argv[optind], MIN(strlen(argv[optind]),WORDSIZE));
    else
        usage();

    prepare_files(fdstruct, input, output);

    mutator(fdstruct);

    close(fdstruct->in);
    close(fdstruct->out);

    free(fdstruct);

    return 0;
}

void usage(void)
{
    char * help="\
\nSyntax: mutator [options] wordlist\
\n\nOptions:\
\n\t-v,\
\n\t-h,\
\n\t-o,     [file] File to write the results\
\n\t-b,     Only \"case\" and \"l33t\" mutations\
\n\t-a,     Only advanced mutations\
\n\t-d,     No append,prepend date\
\n\t-s,     No append specials chars\
\n\t-c,     No append,prepend hardcoded strings";
    
    printf("%s %s by %s email:<%s>\n", PROGRAM, VERSION, AUTHOR, EMAIL);
    printf("%s\n\n",help);

    exit(-1);
}

int read_word(int fd, s_word * w)
{
    char c[WORDSIZE];
    int i=-1,s;

    memset(c,'\0',WORDSIZE);

    do
    {
        i++;
        s=read(fd, &c[i],sizeof(char));
   // }while((c[i]!='\n')&&(s!=0));    
    }while(!isblank(c[i]) && !isspace(c[i]) && s!= -1 && i < WORDSIZE);    
    
//    c[i]='\0';
    w->word = strdup(c);
    w->size = i;
    
    return s;
}

void write_word(int fd, s_word * w)
{
    write(fd,w->word,w->size);
    write(fd, "\n",1);
}

void prepare_files(s_files* f, char* in, char* out)
{
    if((f->in=open(in, O_RDONLY)) == -1)
    {
        perror(in);
        exit(-1);
    }
    
    if((f->out=open(out, O_WRONLY|O_TRUNC|O_CREAT, 0664)) == -1)
    {
        perror(out);
        exit(-1);
    }
}
